#!/usr/bin/perl -w

use Data::Dumper;
use File::stat;
use Storable;
#use XML::Simple;

my $ssh_options="-o KbdInteractiveDevices=no -o PasswordAuthentication=no -o StrictHostKeyChecking=no";

sub check_and_update_cache {
	my $cachefile = shift;
	my $servername = shift;
	my $nmondir = shift;
	my $ssh_idfile = shift;
	my $debug = shift;

	#$debug=3;
	print "checking age of cache-file $cachefile ...\n" if($debug > 1);

	#if file older than 5 minutes
	if(-e "$cachefile.down") {
		if( (time - stat("$cachefile.down")->mtime) > 600 ) {
			unlink("$cachefile.down");
		}
		else {
			return 3;
		}
	}
	if(-e $cachefile) {
		#if( (time - stat($cachefile)->mtime) > 0 ) {
		if( (time - stat($cachefile)->mtime) > 280 ) {
			print "cachefile too old, updating...\n" if($debug > 1);
			return &update_nmon_cache($cachefile,$servername,$nmondir,$ssh_idfile,$debug);
		}
		if( stat($cachefile)->size < 2048) {
			print "cachefile too small, updating...\n" if($debug > 1);
			return &update_nmon_cache($cachefile,$servername,$nmondir,$ssh_idfile,$debug);
		}
	}
	
	else {
		print "cachefile $cachefile does not exist, updating...\n" if($debug > 1);
		return &update_nmon_cache($cachefile,$servername,$nmondir,$ssh_idfile,$debug);
	}
}

sub update_nmon_cache {
	my $cachefile = shift;
	my $servername = shift;
	my $nmondir = shift;
	my $ssh_idfile = shift;
	my $debug = shift;
	my $nmon_tmpdir = shift;
	my %helper;

	my $sshcommand = qq(ssh $ssh_options -i $ssh_idfile nagprobe\@$servername);
	
	$beforecommand = "cat /srv/www/vhosts/cacti/scripts/parse_nmon_cacti.sh | ";
	$command = 'cat \> /tmp/parse_nmon_cacti.sh\; sh /tmp/parse_nmon_cacti.sh';
	my $fullcommand = $beforecommand . " " . $sshcommand . " " . $command . " |";
	print STDERR "FULLCOMMAND: $fullcommand\n" if($debug > 1);

	open(OUTPUT,$fullcommand) || die "Failed: $!\n";
	my $i=0;
	foreach $line ( <OUTPUT> )
	{
		analyze_nmon_line($line,\%helper,$servername);
		$i++;
	}
	if($i < 10) {
		open(OUT,">$cachefile.down");
		print OUT "Host down";
		close OUT;
		unlink($cachefile);
		return 2;
	}
	my $hashref = \%helper;
	delete $hashref->{TMP};
	unlink($cachefile);
	store(\%helper, $cachefile);
#	my $xml = new XML::Simple (NoAttr=>1, RootName=>'data');
#	open(XML,">$cachefile.xml");
#	print XML $xml->XMLout(\%helper);
#	close(XML);
}

sub analyze_nmon_line {
	my $line = shift;
	my $helper = shift;
	my $servername = shift;
	my $debug = 0;
	my @simple_types = ("MEM", "CPU_ALL", "LPAR", "FILE", "PAGE", "PROC", "POOL");
	my @simple_numbered_types = ("CPU");
	my @complex_types = ("DISK","JFS");
	#my @very_complex_types = ("NET", "NETPACKET", "NETSIZE", "NETERROR", "IOADAPT");
	my @very_complex_types = ("NET", "IOADAPT");

	my $type;
	my $description;
	my $attrs_str;
	my $values_str;


	#is line a field description
        if ( ($type,$description,$attrs_str) = $line =~ /^(\w+),(\w+\s[^,]+),(.+)$/) {
		print "FIELD_DESCRIPTION: $type - $description - $attrs_str\n" if($debug > 1);
		my @attrs = split(/,/,$attrs_str);

		#simple, static nmon lines ...	
		#print STDERR Dumper($helper);
		foreach $match_type (@simple_types) {
			if($type =~ /^$match_type/) {
				#fill helper hash with available fields
				my $i=0;
				foreach $attr (@attrs) {
					$helper->{TMP}->{$type}->{$i} = $attr;
					print "category: $type - i: $i - attribute: $attr\n" if($debug > 1);
					$i++;
				}
				last;
			}
		}
		#simple, static nmon lines with numbers ...	
		foreach $match_type (@simple_numbered_types) {
			if($type =~ /^$match_type\d+/) {
				#fill helper hash with available fields
				my $i=0;
				foreach $attr (@attrs) {
					$helper->{TMP}->{$type}->{$i} = $attr;
					print "category: $type - i: $i - attribute: $attr\n" if($debug > 1);
					$i++;
				}
				last;
			}
		}
		#complex types ... 
		foreach $complex_type (@complex_types) {
			if($type =~ /$complex_type/) {
				#print "MATCHING TYPE FIELD: $complex_type\n";
                		#DISKBUSY,Disk %Busy alp2a21,hdisk11,hdisk12,hdisk13,hdisk14,hdisk15,hdisk16,hdisk17,hdisk0
				#
				#get available harddisks and prefill hash
		
				my @disks = split(/,/,$attrs_str);
		
				my $i=0;
				foreach $disk (@disks) {
					$helper->{COMPLEX}->{$complex_type}->{$i}->{device} = $disk;
					$i++;
				}
				last;
			}
		}
		foreach $match_type (@very_complex_types) {
			if($type =~ /^$match_type/) {
				#print "MATCHING TYPE VALUE: $match_type\n";

				my @attrs = split(/,/,$attrs_str);

				my $i=0;
				foreach $attr (@attrs) {
					$helper->{TMP}->{COMPLEX}->{$type}->{fields}->{$i} = $attr;

					print "device: $type - i: $i - attribute: $attr\n" if($debug > 1);

					$i++;

				}
				last;	
			}

		}
	}

	#or a measurement ? 
        elsif ( ($type,$values_str) = $line =~ /^(\w+),T\d{4},(.+)$/) {
		print "MEASUREMENT: $type - $values_str\n" if($debug > 1);
		my @values = split(/,/,$values_str);

		#simple, static nmon lines...
		#print STDERR Dumper($helper);
		foreach $match_type (@simple_types) {
			if($type =~ /^$match_type/) {
				my $i=0;
				#print "$type: ";
				foreach $value (@values) {
					$value = normalize_value($value);
					$attr = $helper->{TMP}->{$type}->{$i};
					print "attr: $attr\n" if($debug > 1);
					
					$attr = normalize_attr($attr);
					#print "$attr:$value ";
					print "attr/value: $attr: $value\n" if($debug > 1);
					if($value ne "") {	
							$helper->{SIMPLE}->{$match_type}->{$attr} = $value;
					}
					$i++;
				}
				#print "\n";
				last;
			}
		}
		#simple but numbered static nmon lines...
		foreach $match_type (@simple_numbered_types) {
			if(my ($elementnum) = $type =~ /^$match_type(\d+)/) {
				$elementnum -= 1;
				my $i=0;
				#print "$type: ";
				foreach $value (@values) {
					$value = normalize_value($value);
					$attr = $helper->{TMP}->{$type}->{$i};
					print "attr: $attr\n" if($debug > 1);
					
					$attr = normalize_attr($attr);
					#print "$attr:$value ";
					print "attr/value: $attr: $value\n" if($debug > 1);
		
					$helper->{COMPLEX}->{$match_type}->{$elementnum}->{device} = $type;
					$helper->{COMPLEX}->{$match_type}->{$elementnum}->{$attr} = $value;
					$i++;
				}
				#print "\n";
				last;
			}
		}
		#complex type,
		foreach $match_type (@complex_types) {
			if($type =~ /$match_type/) {
				#print "MATCHING TYPE VALUE: $match_type\n";
		
				my @values = split(/,/,$values_str);
				$type = lc $type;

				my $i=0;
				foreach $value (@values) {
					$value = normalize_value($value);
					$helper->{COMPLEX}{$match_type}->{$i}->{$type}=$value;
					$i++;
				}
				#print "$match_type:\n";
				last;
			}
		}
		foreach $match_type (@very_complex_types) {
			if($type =~ /^$match_type/) {
				#print "MATCHING TYPE VALUE: $match_type\n";

				my @values = split(/,/,$values_str);

				my $i=0;
				my $j=0;
				my $stats;
				foreach $value (@values) {
					$value = normalize_value($value);
					#print STDERR "type: $type match_type: $match_type helper string: $helper->{TMP}->{COMPLEX}->{$match_type}->{fields}->{$i}\n";
					my ($device,$attribute) = $helper->{TMP}->{COMPLEX}->{$type}->{fields}->{$i} =~ /([\w]{2,8})[-_](.+)/;
	
					print "device: $device - attribute: $attribute - value: $value\n" if($debug > 1);
					
					$attribute = normalize_attr($attribute);
	
					my $exists=0;		
					my $exists_id=0;	
					foreach $key (keys %{$helper->{COMPLEX}->{$match_type}}) {
						if(exists $helper->{COMPLEX}->{$match_type}->{$key}->{device} and $helper->{COMPLEX}->{$match_type}->{$key}->{device} eq $device) {
							print "$device already exists, not adding new entry to hash\n" if($debug > 1);
							$exists = 1;
							$exists_id = $key;
							last;
						}
					}
		
					if($exists == 1) {
						$helper->{COMPLEX}->{$match_type}->{$exists_id}->{$attribute} = $value;
					}
					else {
						$helper->{COMPLEX}->{$match_type}->{$j}->{device} = $device;
						$helper->{COMPLEX}->{$match_type}->{$j}->{$attribute} = $value;
						$j++;
					}
					$i++;
				}
				#print "storing in memcached: $servername - $match_type\n";
				
				#delete $helper->{COMPLEX}->{$type}->{fields};
				last;
			}
		}
	}
			#print Dumper($helper);
}

sub normalize_attr {
	$attr = shift;
	$attr =~ s/\s+$//g;
	$attr =~ s/^\s+//g;
	$attr =~ s/\s/_/g;
	$attr =~ s/\%_/pct/g;
	$attr =~ s/(\w)\%/$1_pct/g;
	$attr =~ s/\%(\w)/pct_$1/g;
	$attr =~ s/\%$/pct/g;
	$attr =~ s/\(MB\)/_mb/g;
	$attr =~ s/\/s/ps/g;
	$attr =~ s/__/_/g;
	$attr = lc($attr);
	return $attr;
}

sub normalize_value {
	$value = shift;
	$value =~ s/\s+$//g;
	$value =~ s/^\s+//g;
	return $value;
}

1;
